# coding=utf-8
import shutil
import subprocess
from traceback import format_exc
import os
import pwd

class check():
    def __init__(self):
        self.func_result = []

    # 检查是否安装了RPC
    def check_rpc_installed(self):
        result = {"value": "update_rpc_installed", "msg": "检查是否安装了RPC"}
        try:
            cmd = "dpkg -s rpcbind"
            proc = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            proc.communicate()
            if proc.returncode == 0:
                 result["code"] = 1
            else:
                result["code"] = 0
        except Exception as ex:
            result["err_msg"] = ex
            result["code"] = 1
        self.func_result.append(result)

    # 检查是否安装了auditd
    def check_auditd_installed(self):
        result = {"value": "update_auditd_installed", "msg": "检查是否安装了auditd"}
        try:
            res = subprocess.Popen(['dpkg', '-s', 'auditd'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            output, error = res.communicate()
            if "install ok installed" in output:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = ex
            result["code"] = 1
        self.func_result.append(result)

    # 检查auditd服务状态
    def check_auditd_service(self):
        result = {"value": "update_auditd_service", "msg": "检查auditd服务状态"}
        try:
            res = subprocess.Popen(['service', 'auditd', 'status'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            output, error = res.communicate()
            if "Active: active" in output:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = ex
            result["code"] = 1
        self.func_result.append(result)

    # 检查审计日志存储大小配置
    def check_auditd_log_size(self):
        result = {"value": "update_auditd_log_size", "msg": "检查审计日志存储大小配置"}
        try:
            p1 = subprocess.Popen(['grep', '-E', '^max_log_file\s*=', '/etc/audit/auditd.conf'], stdout=subprocess.PIPE)
            p2 = subprocess.Popen(['cut', '-d', ' ', '-f3'], stdin=p1.stdout, stdout=subprocess.PIPE)
            p1.stdout.close()
            max_log_file = p2.communicate()[0].strip()
            if max_log_file == "100":
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 检查审计日志自动删除配置
    def check_auditd_auto_delete(self):
        result = {"value": "update_max_log_no_del", "msg": "确保审核日志不会自动删除"}
        try:
            p1 = subprocess.Popen(['grep', '-E', '^max_log_file_action\s*=', '/etc/audit/auditd.conf'], stdout=subprocess.PIPE)
            p2 = subprocess.Popen(['cut', '-d', ' ', '-f3'], stdin=p1.stdout, stdout=subprocess.PIPE)
            p1.stdout.close()
            max_log_file_action = p2.communicate()[0].strip()
            if max_log_file_action == "keep_logs":
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 检查系统在审计日志满时是否被禁用
    def check_auditd_system_disabled(self):
        result = {"value": "update_auditd_system_disabled", "msg": "检查系统在审计日志满时是否被禁用"}
        try:
            p1 = subprocess.Popen(['grep', '-E', '^admin_space_left_action\s*=', '/etc/audit/auditd.conf'],
                                  stdout=subprocess.PIPE)
            p2 = subprocess.Popen(['cut', '-d', ' ', '-f3'], stdin=p1.stdout, stdout=subprocess.PIPE)
            p1.stdout.close()
            admin_space_left_action = p2.communicate()[0].strip()
            if admin_space_left_action == "SUSPEND":
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 检查是否收集修改用户/组信息的事件
    def check_auditd_user_group_events(self):
        result = {"value": "update_user_group_log_sys", "msg": "确保收集了修改用户/组信息的事件"}
        try:
            res = subprocess.Popen(['auditctl', '-l'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            output, error = res.communicate()
            if "/etc/group" in output and "/etc/passwd" in output:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保配置了/etc/ssh/sshd_config的权限
    def check_sshd_config_permissions(self):
        result = {"value": "update_sshd_config_permissions", "msg": "确保配置了/etc/ssh/sshd_config的权限"}
        try:
            output = subprocess.check_output(['ls', '-l', '/etc/ssh/sshd_config'])
            permissions = output.split()[0]
            if permissions == '-rw-r--r--':
                result["code"] = 0
            else:
                result["code"] = 1
        except subprocess.CalledProcessError as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保SSH LogLevel是适当的
    def check_SSH_LogLevel(self):
        result = {"value": "update_SSH_LogLevel", "msg": "确保SSH LogLevel是适当的"}
        try:
            output = subprocess.check_output(['grep', 'LogLevel', '/etc/ssh/sshd_config'])
            output1 = output.split()[0]
            output2 = output.split()[1]
            if '#' not in output1 and 'INFO' == output2:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保SSH MaxAuthTries被设置为4或更少
    def check_max_suer_login(self):
        result = {}
        result["value"] = "update_max_user_loggin"
        result["msg"] = "确保SSH MaxAuthTries设置为4或更低(小)"
        num = 0
        try:
            with open("/etc/ssh/sshd_config") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line.startswith("MaxAuthTries 4"):
                        num += 1
            if num > 0:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    # 确保/etc/shadow和/etc/gshadow文件权限
    def User_And_Group_ShadowFileMode(self):
        result = {}
        result["value"] = "update_group_and_shadow_and_gshadowfilemode"
        result["msg"] = "确保已配置/etc/shadow和/etc/gshadow和/etc/group的权限"
        try:
            # 用户shadow和组shadow文件、备份文件的权限都需要调整
            user_shadow_file = pwd.getpwuid(os.stat('/etc/shadow').st_uid)[0]
            user_shadow_file_limits = os.popen("stat -c %a /etc/shadow").read()
            user_shadow_bakfile = pwd.getpwuid(os.stat('/etc/shadow-').st_uid)[0]
            user_shadow_bakfile_limits = os.popen("stat -c %a /etc/shadow-").read()
            group_shadow_file = pwd.getpwuid(os.stat('/etc/gshadow').st_uid)[0]
            group_shadow_file_limits = os.popen("stat -c %a /etc/gshadow").read()
            group_shadow_bakfile = pwd.getpwuid(os.stat('/etc/gshadow-').st_uid)[0]
            group_shadow_bakfile_limits = os.popen("stat -c %a /etc/gshadow-").read()
            user_group_file = pwd.getpwuid(os.stat('/etc/group').st_uid)[0]
            user_group_file_limits = os.popen("stat -c %a /etc/group").read()
            user_group_bakfile = pwd.getpwuid(os.stat('/etc/group-').st_uid)[0]
            user_group_bakfile_limits = os.popen("stat -c %a /etc/group-").read()
            if user_shadow_file == "root" and int(user_shadow_file_limits.strip('\n')) <= 640 \
                    and group_shadow_file == "root" and int(group_shadow_file_limits.strip('\n')) <= 640 \
                    and user_shadow_bakfile == "root" and int(user_shadow_bakfile_limits.strip('\n')) <= 640 \
                    and group_shadow_bakfile == "root" and int(group_shadow_bakfile_limits.strip('\n')) <= 640\
                    and user_group_file == "root" and int(user_group_file_limits.strip('\n')) <= 640\
                    and user_group_bakfile == "root" and int(user_group_bakfile_limits.strip('\n')) <= 640:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    # 默认用户shell超时为600秒
    def User_Shell_Tmout(self):
        result = {}
        result["value"] = "update_shell_TMOUT"
        result["msg"] = "确保默认用户shell超时为900秒或更短"
        num = 0
        try:
            with open("/etc/profile") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "export TMOUT=600":
                        num += 1
            with open("/etc/bash.bashrc") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "export TMOUT=600":
                        num += 1
            if num == 2:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保/etc/shadow密码字段不为空
    def User_Null_Pass(self):
        result = {}
        result["value"] = "update_user_null_pass"
        result["msg"] = "确保密码字段不为空"
        num = 0
        try:
            with open("/etc/shadow") as f:
                for line in f.readlines():
                    line = line.strip().split(":")
                    if line[1] == "":
                        num += 1
            if num == 0:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保配置了密码创建要求
    def check_password_requirements(self):
        result = {"value": "update_password_requirement", "msg": "确保配置了密码创建要求"}
        try:
            output = subprocess.check_output(['apt', 'list', '--installed', 'libpam-pwquality'])
            if output != 0:
                output1 = subprocess.check_output(['grep', 'minlen = 14', '/etc/security/pwquality.conf'])
                output2 = subprocess.check_output(['grep', 'minclass = 4', '/etc/security/pwquality.conf'])
                if 'minlen = 14' in output1 and 'minclass = 4' in output2:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                result["err_msg"] = "libpam-pwquality未安装"
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保密码的重复使用收到限制
    def Disable_UserPass_Reuse(self):
        result = {}
        result["value"] = "update_disable_userpass_reuse"
        result["msg"] = "确保限制密码重用"
        num = 0
        try:
            with open("/etc/pam.d/common-password") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "password required pam_pwhistory.so remember=5":
                        num += 1
            if num == 1:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保配置了密码尝试失败的锁定功能
    def check_password_lockout(self):
        result = {"value": "update_password_lock", "msg": "确保配置了密码尝试失败的锁定"}
        try:
            output = subprocess.check_output(['grep', 'pam_tally2.so', '/etc/pam.d/common-auth'])
            if 'deny=5' in output:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保配置了更改密码的最短间隔天数
    def check_min_password_age(self):
        result = {"value": "update_min_password_age", "msg": "确保配置了更改密码的最短间隔天数"}
        try:
            output = subprocess.check_output(['grep', 'PASS_MIN_DAYS', '/etc/login.defs'])
            if output.strip().startswith("PASS_MIN_DAYS"):
                min_days = int(output.strip().split()[1])
                if min_days >= 1:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保配置了密码过期时间为365天
    def check_max_password_age(self):
        result = {"value": "update_max_password_age", "msg": "确保配置了密码过期时间为365天以上"}
        try:
            output = subprocess.check_output(['grep', 'PASS_MAX_DAYS', '/etc/login.defs'])
            if output.strip().startswith("PASS_MAX_DAYS"):
                max_days = int(output.strip().split()[1])
                if max_days >= 365:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保不存在重复的UID
    def check_unique_uid(self):
        result = {"value": "update_unique_uid", "msg": "确保不存在重复的UID"}
        try:
            output = subprocess.check_output(['awk', '-F:', '{print $3}', '/etc/passwd'])
            uids = output.strip().split("\n")
            unique_uids = set(uids)
            if len(uids) == len(unique_uids):
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 确保root是唯一UID为0的用户
    def check_unique_root_uid(self):
        result = {"value": "update_unique_root_uid", "msg": "确保root是唯一UID为0的用户"}
        try:
            output = subprocess.check_output(['grep', '^root:', '/etc/passwd'])
            if output.strip().startswith("root:"):
                uid = int(output.strip().split(":")[2])
                if uid == 0:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                result["code"] = 1
        except Exception as ex:
            result["err_msg"] = str(ex)
            result["code"] = 1
        self.func_result.append(result)

    # 检测umask值
    def Check_Umask(self):
        result = {}
        result["value"] = "update_check_umask"
        result["msg"] = "确保默认用户umask限制为027或更高"
        num = 0
        try:
            with open("/etc/profile") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line.startswith("umask"):
                        if int(line.split(" ")[1]) >= 27:
                            num += 1
            with open("/etc/bash.bashrc") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line.startswith("umask"):
                        if int(line.split(" ")[1]) >= 27:
                            num += 1
            if num == 2:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as e:
            result["code"] = 1
        self.func_result.append(result)

    def check_baseline(self):
        try:
            self.check_rpc_installed()
            self.check_auditd_installed()
            self.check_auditd_service()
            self.check_auditd_log_size()
            self.Disable_UserPass_Reuse()
            self.check_auditd_auto_delete()
            self.check_auditd_system_disabled()
            self.check_auditd_user_group_events()
            self.check_sshd_config_permissions()
            self.check_SSH_LogLevel()
            self.check_max_suer_login()
            self.User_And_Group_ShadowFileMode()
            self.User_Shell_Tmout()
            self.User_Null_Pass()
            self.check_password_requirements()
            self.check_password_lockout()
            self.check_min_password_age()
            self.check_max_password_age()
            self.check_unique_uid()
            self.check_unique_root_uid()
            self.Check_Umask()
            return self.func_result
        except:
            print(format_exc())

    def update_option(self, param):
        result = {}
        func_name = param['func_name']
        try:
            agentClass = Modify()
            if hasattr(agentClass, func_name) == True:
                mtd = getattr(agentClass, func_name)
                if mtd() == 0:
                    result["msg"] = func_name
                    result["code"] = 0
                else:
                    result["msg"] = func_name
                    result["code"] = 1
        except:
            result["msg"] = func_name
            result["code"] = 1
        return result


class Modify():
    def __init__(self):
        self.result = ''

    # 修复已安装RPC
    def update_rpc_installed(self):
        try:
            # result = subprocess.Popen(['apt-get', 'purge', '-y', 'rpcbind'], stdout=subprocess.PIPE,
            #                           stderr=subprocess.PIPE)
            # output, error = result.communicate()
            output = os.system('apt-get purge -y rpcbind')
            if output == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复未安装auditd
    def update_auditd_installed(self):
        try:
            # result = subprocess.Popen(['apt-get', 'install', '-y', 'auditd'], stdout=subprocess.PIPE,
            #                           stderr=subprocess.PIPE)
            # output, error = result.communicate()
            output = os.system('apt-get install -y auditd')
            if output == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复auditd服务状态
    def update_auditd_service(self):
        try:
            # result1 = subprocess.Popen(['update-rc.d', 'auditd', 'enable'], stdout=subprocess.PIPE,
            #                            stderr=subprocess.PIPE)
            # output1, error1 = result1.communicate()
            # result2 = subprocess.Popen(['service', 'auditd', 'start'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            # output2, error2 = result2.communicate()
            output1 = os.system("update-rc.d auditd enable")
            output2 = os.system("service auditd start")
            if output1 == 0 and output2 == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复审计日志存储大小配置
    def update_auditd_log_size(self):
        try:
            # result = subprocess.Popen(['sed', '-i', 's/.*max_log_file\s*=.*/max_log_file = 100/g', '/etc/audit/auditd.conf'], stdout=subprocess.PIPE,
            #                            stderr=subprocess.PIPE)
            # output, error = result.communicate()
            shutil.copy("/etc/audit/auditd.conf", "/etc/audit/auditd.conf.bak")
            output = os.system("sed -i 's/.*max_log_file\s*=.*/max_log_file = 100/g' /etc/audit/auditd.conf")
            if output == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复审计日志自动删除配置
    def update_max_log_no_del(self):
        try:
            # result = subprocess.Popen(['sed', '-i', 's/.*max_log_file_action\s*=.*/max_log_file_action = keep_logs/g', '/etc/audit/auditd.conf'], stdout=subprocess.PIPE,
            #                            stderr=subprocess.PIPE)
            # output, error = result.communicate()
            shutil.copy("etc/audit/auditd.conf", "/etc/audit/auditd.conf.bak")
            output = os.system("sed -i 's/.*max_log_file_action\s*=.*/max_log_file_action = keep_logs/g' /etc/audit/auditd.conf")
            if output == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复系统在审计日志满时被禁用
    def update_auditd_system_disabled(self):
        try:
            # result = subprocess.Popen(['sed', '-i', 's/^\s*space_left_action\s*=.*/space_left_action = SUSPEND/g', '/etc/audit/auditd.conf'], stdout=subprocess.PIPE,
            #                            stderr=subprocess.PIPE)
            # output, error = result.communicate()
            shutil.copy("/etc/audit/auditd.conf", "/etc/audit/auditd.conf.bak")
            output = os.system("sed -i 's/.*space_left_action\s*=.*/space_left_action = SUSPEND/g' /etc/audit/auditd.conf")
            if output == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复是否收集修改用户/组信息的事件
    def update_user_group_log_sys(self):
        try:
            # result1 = subprocess.Popen(['auditctl', '-w', '/etc/passwd', '-p', 'wa', '-k', 'identity'], stdout=subprocess.PIPE,
            #                            stderr=subprocess.PIPE)
            # output1, error1 = result1.communicate()
            # result2 = subprocess.Popen(['auditctl', '-w', '/etc/group', '-p', 'wa', '-k', 'identity'], stdout=subprocess.PIPE,
            #                            stderr=subprocess.PIPE)
            # output2, error2 = result2.communicate()
            output1 = os.system("auditctl -w /etc/passwd -p wa -k identity")
            output2 = os.system("auditctl -w /etc/group -p wa -k identity")
            if output1 == 0 and output2 == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复/etc/ssh/sshd_config的权限
    def update_sshd_config_permissions(self):
        try:
            # output = subprocess.check_output(['chown', 'root:root', '/etc/ssh/sshd_config'])
            # permissions = output.split()[0]
            output = os.system("chown root:root /etc/ssh/sshd_config")
            if output == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复SSH_LogLevel为适当的
    def update_SSH_LogLevel(self):
        try:
            result1 = os.system("/usr/bin/cp /etc/ssh/sshd_config /etc/ssh/sshd_config.bak")
            result2 = os.system(
                "sed -i 's/.*LogLevel.*/LogLevel INFO/g' /etc/ssh/sshd_config && systemctl restart sshd")
            if result1 == 0 and result2 == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复SSH MaxAuthTries被设置为4或更少
    def update_max_user_loggin(self):
        try:
            result1 = os.system("/usr/bin/cp /etc/ssh/sshd_config /etc/ssh/sshd_config.bak")
            result2 = os.system(
                "sed -i '/.*MaxAuthTries/d' /etc/ssh/sshd_config && echo 'MaxAuthTries 4' >> /etc/ssh/sshd_config && systemctl restart sshd")
            if result1 == 0 and result2 == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复/etc/shadow /etc/shadow- /etc/gshadow /etc/gshadow-  /etc/group /etc/group-文件权限
    def update_group_and_shadow_and_gshadowfilemode(self):
        try:
            result1 = os.system("chown root:root /etc/shadow && chmod u-x,g-wx,o-rwx /etc/shadow && chown root:root /etc/shadow- && chmod u-x,g-wx,o-rwx /etc/shadow-")
            result2 = os.system("chown root:root /etc/gshadow && chmod u-x,g-wx,o-rwx /etc/gshadow && chown root:root /etc/gshadow-  && chmod u-x,g-wx,o-rwx /etc/gshadow-")
            result3 = os.system("chown root:root /etc/group && chmod u-x,g-wx,o-rwx /etc/group && chown root:root /etc/group-  && chmod u-x,g-wx,o-rwx /etc/group-")
            if result1 == 0 and result2 == 0 and result3 == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复shell连接超时未设置600s
    def update_shell_TMOUT(self):
        try:
            shutil.copy("/etc/profile", "/etc/profile.bak")
            shutil.copy("/etc/bash.bashrc", "/etc/bash.bashrc.bak")
            result1 = os.system("""sed -i '/.*export TMOUT\|.*TMOUT/d' /etc/bash.bashrc && sed -i '/.*export TMOUT\|.*TMOUT/d' /etc/profile""")
            result2 = os.system("""echo 'export TMOUT=600' >> /etc/profile && echo 'export TMOUT=600' >> /etc/bash.bashrc""")
            if result1 == 0 and result2 == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复/etc/shadow密码字段为空
    def update_user_null_pass(self):
        try:
            with open("/etc/shadow") as f:
                for line in f.readlines():
                    line = line.strip().split(":")
                    if line[1] == "":
                        user_name = line[0]
                        if os.system("passwd -l %s" %user_name) == 0:
                            return 0
                        else:
                            return 1
                    else:
                        return 0
        except:
            return 1

    # 修复密码重用
    def update_disable_userpass_reuse(self):
        try:
            shutil.copy("/etc/pam.d/common-password", "/etc/pam.d/common-password.bak")
            result = os.system("""sed -i '/.*password.*required.*pam_pwhistory.so.*remember=.*/d' /etc/pam.d/common-password  && echo 'password required pam_pwhistory.so remember=5' >> /etc/pam.d/common-password""")
            if result == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复密码创建要求
    def update_password_requirement(self):
        try:
            shutil.copy("/etc/security/pwquality.conf", "/etc/security/pwquality.conf.bak")
            result1 = os.system("sed -i '/.*minlen/d' /etc/security/pwquality.conf && echo 'minlen = 14' >> /etc/security/pwquality.conf")
            result2 = os.system("sed -i '/.*minclass/d' /etc/security/pwquality.conf && echo 'minclass = 4' >> /etc/security/pwquality.conf")
            if result1 == 0 and result2 == 0 :
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复密码尝试五次冻结900s
    def update_password_lock(self):
        try:
            shutil.copy("/etc/pam.d/common-auth", "/etc/pam.d/common-auth.bak")
            result = os.system(
                """sed -i '/.*password.*required.*pam_tally2.so.*deny=.*/d' /etc/pam.d/common-auth && echo 'password required pam_tally2.so deny=5 unlock_time=900' >> /etc/pam.d/common-auth""")
            if result == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复密码更改最短间隔天数
    def update_min_password_age(self):
        try:
            shutil.copy("/etc/login.defs", "/etc/login.defs.bak")
            # result = subprocess.call(["sed", "-i", "s/.*PASS_MIN_DAYS.*/PASS_MIN_DAYS 1/", "/etc/login.defs"])
            result = os.system("sed -i 's/.*PASS_MIN_DAYS.*/PASS_MIN_DAYS 1/g' /etc/login.defs")
            if result == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复密码过期时间为365天
    def update_max_password_age(self):
        try:
            shutil.copy("/etc/login.defs", "/etc/login.defs.bak")
            # result = subprocess.call(["sed", "-i", "s/.*PASS_MAX_DAYS.*/PASS_MAX_DAYS 99999/g", "/etc/login.defs"])
            result = os.system("sed -i 's/.*PASS_MAX_DAYS.*/PASS_MAX_DAYS 99999/g' /etc/login.defs")
            if result == 0:
                return 0
            else:
                return 1
        except Exception as ex:
            return 1

    # 修复UID不唯一
    def update_unique_uid(self):
        try:
            # 从/etc/passwd文件中获取所有用户的UID
            uid_list = [int(line.split(":")[2]) for line in open("/etc/passwd")]
            # 找出重复的UID
            duplicate_uids = []
            for uid in uid_list:
                if uid_list.count(uid) > 1:
                    duplicate_uids.append(uid)
                    duplicate_uids = list(set(duplicate_uids))
                    for uid in duplicate_uids:
                        # 获取重复UID对应的用户名
                        duplicate_users = [user.pw_name for user in pwd.getpwall() if user.pw_uid == uid]
                        # 生成一个新的不重复的UID
                        new_uid = max(uid_list) + 1
                        for user in duplicate_users:
                            # 修改重复UID对应用户的UID
                            uids = str(new_uid)
                            a = "usermod -u " + uids + " " + user
                            result = os.system(a)
                            if result == 0:
                                new_uid += 1
                                return 0
                            else:
                                return 1
                else:
                    return 0
        except:
            return 1

    # 修复root为唯一uid 0的账户
    def update_unique_root_uid(self):
        try:
            result = os.system("usermod -u 0 root")
            if result == 0:
                return 0
            else:
                return 1
        except:
            return 1

    # 修复umask值
    def update_check_umask(self):
        try:
            os.system("""sed -i '/^umask/d' /etc/bash.bashrc && echo 'umask 027' >>/etc/bash.bashrc """)
            result1 = os.system("""sed -i '/^umask/d' /etc/profile && echo 'umask 027' >>/etc/profile &&/bin/bash /etc/profile""")
            if result1 == 0:
                return 0
            else:
                return 1
        except:
            return 1

if __name__ == "main":
    pass
